%% Core Loss Calculation Script

% David Meeker
% dmeeker@ieee.org
% September 7, 2009


%% Parameters

% base frequency in Hz
wbase=5000/60; %(5000 rev/minute)*(minute/(60*seconds))

% Hysteresis coefficient in (Watts/(meter^3 * T^2 * Hz)
ch = 143.; 

% Eddy current coefficient in (Watt/(meter^3 * T^2 * Hz^2)
ce = 0.132;

% Lamination stacking factor (nondimensional)
cs = 0.92;

% Density of stator iron in kg/m^3
rho = 7700.;

% angle through which to spin the rotor in degrees
n = 360;

% angle increment in degrees
dk = 1;

% coil current amplitude
iamp = 20.;

%% Perform a series of finite element analyses

openfemm;
opendocument('rolled.fem');
mi_saveas('temp.fem');

% Run an analysis through an entire spin of the rotor and record
% element centroid flux density (using the mesh from the first iteration)
% at every step.  This information will then be used to estimate
% core losses
for k = 0:dk:(n-1)
	starttime=clock;
    kk = k/dk + 1;
    
    % make sure that the current is set to the appropriate value for this iteration.  
    mi_modifycircprop('icoil', 1, iamp*cos(6*k*pi/180));
    
    mi_analyze(1);
    mi_loadsolution;
    if (k == 0)
        % Record the initial mesh elements if the first time through the loop
        nn = mo_numelements;
        b = zeros(floor(n/dk),nn); %matrix that will hold the flux density info
        z = zeros(nn,1);
        a = zeros(nn,1);
        g = zeros(nn,1);
        tq = zeros(floor(n/dk),1);
        for m = 1:nn
            elm = mo_getelement(m);
            % z is a vector of complex numbers that represents the location of
            % the centroid of each element.  The real part is x, the
            % imaginary part is y.  The purpose of representing the
            % location in this way is that it is easy to rotate the
            % location of the centroid (by multiplying by a complex number)
            % to find the point that corresponds to the centroid when the
            % rotor is rotated.
            z(m) = elm(4) + j*elm(5);
            % element area in the length units used to draw the geometry
            a(m) = elm(6);
            % group number associated with the element
            g(m) = elm(7);
        end
    end
    
    % Store element flux densities *)
    u=exp(j*k*pi/180.);
    for m = 1:nn
        % Element is on the rotor.  Elements on the rotor have been
        % assigned to group 1 (by assigning the block label that marks them
        % to group 1) so that the program can tell if an element is part of the rotor. 
        if(g(m)==1) 
            % the location in the original mesh is rotated so that the
            % flux density at the same point with the rotor rotated through
            % angle k can be evaluated. Multiplying by the complex number u
            % does the rotation.
            p = z(m)*u;
            % Flux densities bx and by are evaluated and rolled into a complex number.
            % Dividing by the complex number u rotates the flux density
            % back into a rotor-fixed reference frame.
            b(kk,m) = (mo_getb(real(p),imag(p))*[1;j])/u;
        end
        % element is on the stator
        if(g(m) == 2) 
            % since the stator doesn't move, no games have to be played
            % with rotations.
            p = z(m);
            b(kk,m) = (mo_getb(real(p),imag(p))*[1;j]);
        end
    end

    % mo_getprobleminfo returns, among other things, the depth of the
    % machine in the into-the-page direction and the length units used to
    % draw the geometry. Both of these pieces of information will be needed
    % to integrate the losses over the volume of the machine.
    probinfo=mo_getprobleminfo;
    
    mo_groupselectblock(1);
    tq(kk)=mo_blockintegral(22);
    mo_close;
    
    % rotate the rotor to the position for the next iteration.
    mi_selectgroup(1);
    mi_moverotate(0, 0, dk);     
    
    disp(sprintf('%i of %i :: %f seconds',k,n,etime(clock,starttime)));
end

% clean up after finite element runs are finished
closefemm;
delete('temp.fem');
delete('temp.ans');

%% Add Up Core Losses

% Compute the square of the amplitude of each harmonic at the centroid of
% each element in the mesh. Matlab's built-in FFT function makes this easy.
bxfft=abs(fft(real(b)))*(2/n);
byfft=abs(fft(imag(b)))*(2/n);
bsq=(bxfft.*bxfft) + (byfft.*byfft);

% Make a vector representing the frequency associated with each harmonic
% The last half of the entries are zeroed out so that we don't count each
% harmonic twice--the upper half of the FFT a mirror of the lower half
w=0:(n-1);
w=wbase*w.*(w<(n/2)); 

% Compute the volume of each element in units of meter^3
h = probinfo(3);            %Length of the machine in the into-the-page direction
lengthunits = probinfo(4);  %Length of drawing unit in meters
v = a*h*lengthunits^2;

% Now, total core loss can be computed in one fell swoop...
% Dividing the result by cs corrects for the lamination stacking factor .
total_loss = ((ch*w+ce*w.*w)*bsq*v)/cs;

%% Display Results

disp(sprintf('total core loss = %g W',total_loss));

iron_mass = cs*rho*sum(v);
disp(sprintf('loss/volume = %g W/lb',total_loss/(iron_mass*2.20462262)));

disp(sprintf('average torque = %g N*m',mean(tq)));

disp(sprintf('mechanical power = % g W',mean(tq)*wbase*2*pi));

plot(0:dk:(n-1),tq);
xlabel('Rotor Angle, Degrees');
ylabel('Torque, N*m');
title('Torque on Rotor vs. Angle');
